import unittest
from cy_ioc_extract import IOCEXtract
from cy_ioc_extract.exception import UnsupportedExtractFieldError


class TestIOCExtract(unittest.TestCase):
    def setUp(self):
        self.sample_text = """
        ### IPv4 Addresses:
        192.168.1.1
        10.0.0.1
        8.8.8.8
        172.16.32.45

        ### Domains:
        example.com
        subdomain.test.org
        my-site.net
        web.co.uk

        ### Email Addresses:
        john.doe@example.com
        alice_smith@corporate.org
        user123@test.net
        contact@web.co.uk
        """

    def test_extract_specific_fields(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("DOMAIN", "EMAIL"))
        result = extractor.extract_ioc()
        expected = {
            "DOMAIN": ["example.com", "subdomain.test.org", "my-site.net", "web.co.uk"],
            "EMAIL": [
                "john.doe@example.com",
                "alice_smith@corporate.org",
                "user123@test.net",
                "contact@web.co.uk"
            ]
        }
        self.assertEqual(result, expected)

    def test_extract_all_fields(self):
        extractor = IOCEXtract(self.sample_text)
        result = extractor.extract_ioc()
        self.assertIn("IP", result)
        self.assertIn("DOMAIN", result)
        self.assertIn("EMAIL", result)

    def test_extract_without_validation(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("DOMAIN",), validate_ioc=False)
        result = extractor.extract_ioc()
        expected = {"DOMAIN": ["192.168", "example.com", "subdomain.test.org", "my-site.net", "web.co.uk"]}
        self.assertEqual(result, expected)

    def test_extract_invalid_field(self):
        with self.assertRaises(UnsupportedExtractFieldError):
            IOCEXtract(self.sample_text, extract_fields=("DOMAIN", "INVALID_FIELD")).extract_ioc()

    def test_extract_ip_addresses(self):
        extractor = IOCEXtract(self.sample_text, extract_fields=("IP",))
        result = extractor.extract_ioc()
        expected = {"IP": ["192.168.1.1", "10.0.0.1", "8.8.8.8", "172.16.32.45"]}
        self.assertEqual(result, expected)

    def test_extract_empty_text(self):
        extractor = IOCEXtract("")
        result = extractor.extract_ioc()
        for key, val in result.items():
            self.assertEqual(len(val), 0)

    def test_extract_urls(self):
        sample_text = "Check this site: https://example.com and ftp://192.168.1.1/resource"
        extractor = IOCEXtract(sample_text, extract_fields=("URL",))
        result = extractor.extract_ioc()
        expected = {"URL": [("https://example.com", ""), ("ftp://192.168.1.1/resource", "")]}
        self.assertEqual(result, expected)

    def test_extract_hashes(self):
        sample_text = "SHA256: d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2"
        extractor = IOCEXtract(sample_text, extract_fields=("SHA256",))
        result = extractor.extract_ioc()
        expected = {"SHA256": ["d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2d2"]}
        self.assertEqual(result, expected)


if __name__ == "__main__":
    unittest.main()
